GOOGLE_DOCSTRING_ELEMENTS = [
    "Args:",
    "Attributes:",
    "Examples:",
    "Raises:",
    "Returns:",
    "Yields:",
]


REMOVE_ITEMS = [
    "__builtins__",
    "__cached__",
    "__doc__",
    "__file__",
    "__loader__",
    "__name__",
    "__package__",
    "__spec__",
    "__init__",
    "__pycache__",
    "__init__.py",
    "__version__.py",
]
